package rpc

import (
	"context"

	"git.loafle.net/commons_go/rpc"
	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol"
	"git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newGatewayRPCServletHandler() GatewayRPCServletHandler {
	sh := &GatewayRPCServletHandlers{}
	sh.RegisterCodec(json.Name, json.NewServerCodec())
	return sh
}

type GatewayRPCServletHandlers struct {
	gateway.ServletHandlers
}

func (sh *GatewayRPCServletHandlers) Invoke(servletCTX rpc.ServletContext, requestCodec protocol.RegistryCodec) (result interface{}, err error) {
	md := metadata.Pairs(oogw.GRPCUserIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string))
	grpcCTX := metadata.NewOutgoingContext(context.Background(), md)

	params, err := requestCodec.Params()
	if nil != err {
		return nil, err
	}

	r, err := grpc.Exec(grpcCTX, requestCodec.Method(), params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (sh *GatewayRPCServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

}
